from distutils.core import setup
setup(
  name = 'elastic_helper',
  packages = ['elastic_helper'], # this must be the same as the name above
  version = '1.2.2',
  description = 'Elastic Search to Pandas Dataframe',
  author = 'snuids',
  author_email = 'snuids@mannekentech.com',
  url = 'https://github.com/snuids/elastic_helper', 
  download_url = 'https://github.com/snuids/elastic_helper/archive/1.2.2.tar.gz',
  keywords = ['ElasticSearch', 'pandas', 'convert'], # arbitrary keywords
  classifiers = [],
)
